//
//  AppDelegate.m
//  Thunder Tracker
//
//  Created by Kyle McElyea on 3/31/12.
//  Copyright (c) 2012 OmNomRobot. All rights reserved.
//

#import "AppDelegate.h"
#import "CustomView.h"
#import "MAAttachedWindow.h"
#import "Feed.h"
#import "FeedItem.h"
#import "BDLinkArrowCell.h"

// Update every 10 minutes.
static int UPDATE_FEED_INTERVAL = 600;
#define DEFAULTS_KEY @"feeds"

@implementation AppDelegate
@dynamic statusMenu;
@synthesize RSSPop;
@synthesize UpcomingPop;
@synthesize StatPop;
@synthesize StatWebView;
@synthesize ScorePop;
@synthesize ScoreWebView;

- (id) init
{
	self = [super init];
	if (self != nil)
	{
		_isUpdatingFeeds = NO;
		
		// Trigger update timer periodically.
		[NSTimer scheduledTimerWithTimeInterval:UPDATE_FEED_INTERVAL
										 target:self
									   selector:@selector(updateFeeds:)
									   userInfo:nil
										repeats:YES];
        StatWebView = [[WebView alloc] init];
	}
	return self;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Check user defaults.
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	NSArray *defaultFeeds = [defaults arrayForKey:DEFAULTS_KEY];
	if (defaultFeeds == nil)
	{
		// No stored defaults. Use the default ones.
		defaultFeeds = [NSArray arrayWithObjects:@"http://feeds.pheedo.com/feedout/syndicatedContent_categoryId_71099", nil];
	}
	
	// Add default feeds.
	NSEnumerator *feedEnumerator = [defaultFeeds objectEnumerator];
	NSString *feedString;
	while (feedString = [feedEnumerator nextObject])
	{
		[self addFeedToList:feedString];
	}
    
	// Update.
	[self updateFeeds:nil];
	[sourceList selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
}

-(void)awakeFromNib{
    [attachedWindow setLevel:NSMainMenuWindowLevel];
    float width = 25.0;
    float height = [[NSStatusBar systemStatusBar] thickness];
    NSRect viewFrame = NSMakeRect(0, 0, width, height);
    statusItem = [[NSStatusBar systemStatusBar] statusItemWithLength:width];
    
    [statusItem setView:[[CustomView alloc] initWithFrame:viewFrame controller:self]];

    
    [StatWebView setMainFrameURL:@"http://m.espn.go.com/nba/standings"];    
    [ScoreWebView setMainFrameURL:@"http://m.espn.go.com/nba/scoreboard"]; 
    [ThunderStats setMainFrameURL:@"http://m.espn.go.com/nba/clubhouse?teamId=25"]; 
}
- (void)toggleAttachedWindowAtPoint:(NSPoint)pt
{
    // Attach/detach window.
    if (!attachedWindow) {
        attachedWindow = [[MAAttachedWindow alloc] initWithView:view 
                                                attachedToPoint:pt 
                                                       inWindow:nil 
                                                         onSide:MAPositionBottom 
                                                     atDistance:5.0];
        [attachedWindow makeKeyAndOrderFront:self];
    } else {
        [attachedWindow orderOut:self];

        attachedWindow = nil;
    }    
}

-(IBAction)RSSPop:(id)sender{
    
    
    [[self RSSPop] showRelativeToRect:RSSButton.bounds ofView:sender preferredEdge:NSMinXEdge];
}
-(IBAction)UpcomingPop:(id)sender{
    [ThunderStats setMainFrameURL:@"http://m.espn.go.com/nba/clubhouse?teamId=25"]; 
    [[self UpcomingPop] showRelativeToRect:UpcomingButton.bounds ofView:sender preferredEdge:NSMinXEdge];

}
-(IBAction)StatPop:(id)sender{
    
    [StatWebView setMainFrameURL:@"http://m.espn.go.com/nba/standings"];
    [[self StatPop] showRelativeToRect:StatButton.bounds ofView:sender preferredEdge:NSMinXEdge];
    
}

-(IBAction)ScorePop:(id)sender{
    
    [ScoreWebView setMainFrameURL:@"http://m.espn.go.com/nba/scoreboard"];
    [[self ScorePop] showRelativeToRect:ScoreButton.bounds ofView:sender preferredEdge:NSMinXEdge];
    
    
}

-(IBAction)ThunderClick:(id)sender {
    [[NSWorkspace sharedWorkspace] openURL: [NSURL URLWithString:@"http://www.nba.com/thunder/"]];
    
}

//RSS Stuff
- (IBAction)openURLInBrowser:(id)sender
{
	if ([[feedItems selectedObjects] count] > 0)
	{
		// Load selected item into WebView.
		FeedItem *selectedItem = [[feedItems selectedObjects] objectAtIndex:0];
		NSURL *url = [[[selectedItem properties] objectForKey:@"url"] copy];
		[[NSWorkspace sharedWorkspace] openURL:url];
	}
}

- (IBAction)updateFeeds:(id)sender
{
	// Ensure the update thread is a singleton.
	if (_updateThread == nil || [_updateThread isFinished])
	{
		_updateThread = [[NSThread alloc] initWithTarget:self selector:@selector(_updateFeeds) object:nil];
		[_updateThread start];
	}
}

//
// Methods
//

- (void)_updateFeeds
{
	
    
	NSLog(@"updating feeds");
	NSArray *feedArray = [feeds arrangedObjects];
	NSEnumerator *feedEnumerator = [feedArray objectEnumerator];
	Feed *feed;
	while (feed = [feedEnumerator nextObject])
	{
		[feed updateFeed];
	}
	

}

- (BOOL)addFeedToList:(NSString *)feedString
{
	NSURL *feedUrl = [NSURL URLWithString:feedString];
	Feed *feed = [[Feed alloc] initWithUrl:feedUrl];
	if (feed != nil)
	{
		[feeds addObject:feed];
		[sourceList reloadData];
		return YES;
	}
	else
	{
		return NO;
	}
}

- (void)loadUrlIntoWebView:(NSURL *)url
{
	[[webView mainFrame] loadRequest:[NSURLRequest requestWithURL:url]];
}

- (void)updateWebView
{
	if ([[feedItems selectedObjects] count] > 0)
	{
		// Load selected item into WebView.
		FeedItem *selectedItem = [[feedItems selectedObjects] objectAtIndex:0];
		NSURL *url = [[selectedItem properties] objectForKey:@"url"];
		[self loadUrlIntoWebView:url];
	}
}


//
// TableView delegates
//

- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell forTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
	if ([[aTableColumn identifier] isEqualToString:@"Title"])
	{
		// Set title.
		FeedItem *currentFeedItem = [[feedItems arrangedObjects] objectAtIndex:rowIndex];
		[aCell setTitle:[currentFeedItem valueForKeyPath:@"properties.title"]];
		
		// Set link arrow visibility.
		[aCell setLinkVisible:([aTableView selectedRow] == rowIndex)];
	}
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
	[self updateWebView];
}

//
// SplitView delegates
//

- (CGFloat)splitView:(NSSplitView *)sender constrainMaxCoordinate:(CGFloat)proposedMax ofSubviewAt:(NSInteger)offset
{
	if (sender == sourceListSplitView && offset == 0)
	{
		return 300;
	}
	else
	{
		return proposedMax;
	}
}

- (CGFloat)splitView:(NSSplitView *)sender constrainMinCoordinate:(CGFloat)proposedMin ofSubviewAt:(NSInteger)offset
{
	if (sender == sourceListSplitView && offset == 0)
	{
		return 150;
	}
	else if (sender == itemsSplitView && offset == 0)
	{
		return 100;
	}
	else
	{
		return proposedMin;
	}
}


/// Score pop


@end
